<?php  // $Id: report.php,v 1.13.2.2 2008/05/01 07:36:53 jamiesensei Exp $
/**
 * This script lists student attempts and responses
 *
 * @version $Id: report.php,v 1.13.2.2 2008/05/01 07:36:53 jamiesensei Exp $

 * @author Jean-Michel V�drine, Gustav Delius and others.
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package quiz
 */

require_once($CFG->libdir.'/tablelib.php');

class quiz_report extends quiz_default_report {

    /**
     * Display the report.
     */
    function display($quiz, $cm, $course) {
        global $CFG, $SESSION, $db, $QTYPES;

        // Define some strings
        $strnoattempts = get_string('noattempts','quiz');
        $strnoattemptsonly = get_string('shownoattemptsonly', 'quiz');
        $strattemptsonly = get_string('attemptsonly','quiz');
        $strbothattempts = get_string('bothattempts','quiz');
        $strtimeformat = get_string('strftimedatetime');
        $strreviewquestion = get_string('reviewresponse', 'quiz');

        // Only print headers if not asked to download data
        if (!$download = optional_param('download', NULL)) {
            $meta = '<link rel="stylesheet" type="text/css" href="'.$CFG->wwwroot.'/mod/quiz/report/responses/styles.css" />'."\n";
            $this->print_header_and_tabs($cm, $course, $quiz, $reportmode="responses", $meta);
        }
        // Set of format options for teacher-created content, for example overall feedback.
        $nocleanformatoptions = new stdClass;
        $nocleanformatoptions->noclean = true;

        // Set table options
        $noattempts = optional_param('noattempts', 0, PARAM_INT);
        $pagesize = optional_param('pagesize', QUIZ_REPORT_DEFAULT_PAGE_SIZE, PARAM_INT);
        $reporturl = $CFG->wwwroot.'/mod/quiz/report.php?mode=responses';
        $reporturlwithoptions = $reporturl . '&amp;id=' . $cm->id . '&amp;noattempts=' . $noattempts .
                '&amp;pagesize=' . $pagesize;

        // Print information on the number of existing attempts
        if (!$download) { //do not print notices when downloading
            if ($attemptnum = count_records('quiz_attempts', 'quiz', $quiz->id, 'preview', 0)) {
                $a = new stdClass;
                $a->attemptnum = $attemptnum;
                $a->studentnum = count_records_select('quiz_attempts', "quiz = '$quiz->id' AND preview = '0'", 'COUNT(DISTINCT userid)');
                $a->studentstring = $course->students;
    
                notify(get_string('numattempts', 'quiz', $a));
            }
        }

        /// find out current groups mode
        $currentgroup = groups_get_activity_group($cm, true);
        
        if ($groupmode = groups_get_activity_groupmode($cm)) {   // Groups are being used
            if (!$download) {
                groups_print_activity_menu($cm, $reporturlwithoptions);
            }
        }

        // Now check if asked download of data
        if ($download) {
            $filename = clean_filename("$course->shortname ".format_string($quiz->name,true));
            $sort = '';
            $limit = '';
        }

    //  Define table columns
        $tablecolumns = array('picture', 'fullname');
        $tableheaders = array('', get_string('fullname'));
    
        if ($quiz->grade and $quiz->sumgrades) {
            $tablecolumns[] = 'sumgrades';
            $tableheaders[] = get_string('grade', 'quiz').'/'.$quiz->grade;
        }
    
        // we want to display marks for all questions
        // Start by getting all questions
        $questionlist = quiz_questions_in_quiz($quiz->questions);
        $questionids = explode(',', $questionlist);
        $sql = "SELECT q.*, i.grade AS maxgrade, i.id AS instance".
               "  FROM {$CFG->prefix}question q,".
               "       {$CFG->prefix}quiz_question_instances i".
               " WHERE i.quiz = '$quiz->id' AND q.id = i.question".
               "   AND q.id IN ($questionlist)";
        if (!$questions = get_records_sql($sql)) {
            error('No questions found');
        }
        $number = 1;
        foreach($questionids as $key => $id) {
            if ($questions[$id]->length) {
                // Only print questions of non-zero length
                $tablecolumns[] = '$'.$id;
                $tableheaders[] = '#'.$number;
                $questions[$id]->number = $number;
                $number += $questions[$id]->length;
            } else {
                // get rid of zero length questions
                unset($questions[$id]);
                unset($questionids[$key]);
            }
        }
        // Load the question type specific information
        if (!get_question_options($questions)) {
            error('Could not load question options');
        }

        if (!$download) {
            // Set up the table
    
            $table = new flexible_table('mod-quiz-report-responses');
    
            $table->define_columns($tablecolumns);
            $table->define_headers($tableheaders);
            $table->define_baseurl($reporturlwithoptions);

            $table->sortable(true);
            $table->collapsible(true);

            $table->column_suppress('picture');
            $table->column_suppress('fullname');

            $table->column_class('picture', 'picture');

            $table->set_attribute('cellspacing', '0');
            $table->set_attribute('id', 'responses');
            $table->set_attribute('class', 'generaltable generalbox');

            // Start working -- this is necessary as soon as the niceties are over
            $table->setup();
        } else if ($download =='ODS') {
            require_once("$CFG->libdir/odslib.class.php");

            $filename .= ".ods";
            // Creating a workbook
            $workbook = new MoodleODSWorkbook("-");
            // Sending HTTP headers
            $workbook->send($filename);
            // Creating the first worksheet
            $sheettitle = get_string('reportresponses','quiz_responses');
            $myxls =& $workbook->add_worksheet($sheettitle);
            // format types
            $format =& $workbook->add_format();
            $format->set_bold(0);
            $formatbc =& $workbook->add_format();
            $formatbc->set_bold(1);
            $formatbc->set_align('center');
            $formatb =& $workbook->add_format();
            $formatb->set_bold(1);
            $formaty =& $workbook->add_format();
            $formaty->set_bg_color('yellow');
            $formatc =& $workbook->add_format();
            $formatc->set_align('center');
            $formatr =& $workbook->add_format();
            $formatr->set_bold(1);
            $formatr->set_color('red');
            $formatr->set_align('center');
            $formatg =& $workbook->add_format();
            $formatg->set_bold(1);
            $formatg->set_color('green');
            $formatg->set_align('center');
            // Here starts workshhet headers

            $headers = array(get_string('firstname'), get_string('lastname'), get_string('idnumber'), 
			                 get_string('institution'), get_string('department'), get_string('email'),
							 get_string('startedon', 'quiz'), get_string('timecompleted', 'quiz'), get_string('attemptduration', 'quiz'));

            if ($quiz->grade and $quiz->sumgrades) {
                $headers[] = get_string('grade', 'quiz').'/'.$quiz->grade;
            }
            foreach ($questionids as $id) {
                $headers[] = '#'.$questions[$id]->number;
            }
            $colnum = 0;
            foreach ($headers as $item) {
                $myxls->write(0,$colnum,$item,$formatbc);
                $colnum++;
            }
            $rownum=1;
        } else if ($download =='Excel') {
            require_once("$CFG->libdir/excellib.class.php");

            $filename .= ".xls";
            // Creating a workbook
            $workbook = new MoodleExcelWorkbook("-");
            // Sending HTTP headers
            $workbook->send($filename);
            // Creating the first worksheet
            $sheettitle = get_string('reportresponses','quiz_responses');
            $myxls =& $workbook->add_worksheet($sheettitle);
            // format types
            $format =& $workbook->add_format();
            $format->set_bold(0);
            $formatbc =& $workbook->add_format();
            $formatbc->set_bold(1);
            $formatbc->set_align('center');
            $formatb =& $workbook->add_format();
            $formatb->set_bold(1);
            $formaty =& $workbook->add_format();
            $formaty->set_bg_color('yellow');
            $formatc =& $workbook->add_format();
            $formatc->set_align('center');
            $formatr =& $workbook->add_format();
            $formatr->set_bold(1);
            $formatr->set_color('red');
            $formatr->set_align('center');
            $formatg =& $workbook->add_format();
            $formatg->set_bold(1);
            $formatg->set_color('green');
            $formatg->set_align('center');
            // Here starts workshhet headers

            $headers = array(get_string('firstname'), get_string('lastname'), get_string('idnumber'), 
			                 get_string('institution'), get_string('department'), get_string('email'),
							 get_string('startedon', 'quiz'), get_string('timecompleted', 'quiz'), get_string('attemptduration', 'quiz'));

            if ($quiz->grade and $quiz->sumgrades) {
                $headers[] = get_string('grade', 'quiz').'/'.$quiz->grade;
            }
            foreach ($questionids as $id) {
                $headers[] = '#'.$questions[$id]->number;
            }
            $colnum = 0;
            foreach ($headers as $item) {
                $myxls->write(0,$colnum,$item,$formatbc);
                $colnum++;
            }
            $rownum=1;
        } else if ($download=='CSV') {
            $filename .= ".txt";

            header("Content-Type: application/download\n");
            header("Content-Disposition: attachment; filename=\"$filename\"");
            header("Expires: 0");
            header("Cache-Control: must-revalidate,post-check=0,pre-check=0");
            header("Pragma: public");

            $headers = get_string('firstname')."\t".get_string('lastname')."\t".get_string('idnumber')
			                 ."\t".get_string('institution')."\t".get_string('department')."\t".get_string('email')
							 ."\t".get_string('startedon', 'quiz')."\t".get_string('timecompleted', 'quiz')."\t".get_string('attemptduration', 'quiz');

            if ($quiz->grade and $quiz->sumgrades) {
                $headers .= "\t".get_string('grade', 'quiz')."/".$quiz->grade;
            }
            foreach($questionids as $id) {
                $headers .= "\t#".$questions[$id]->number;
            }
            echo $headers." \n";

        }

        $contextlists = get_related_contexts_string(get_context_instance(CONTEXT_COURSE, $course->id));

        // Construct the SQL
        $select = 'SELECT '.sql_concat('u.id', '\'#\'', $db->IfNull('qa.attempt', '0')).' AS attemptindex, '.
            'qa.uniqueid, qa.id AS attempt, u.id AS userid, u.firstname, u.lastname, u.idnumber, '.
			'u.institution, u.department, u.email, u.picture, qa.timefinish - qa.timestart AS duration, qa.*';

        if ($course->id != SITEID) { // this is too complicated, so just do it for each of the four cases.
            if (!empty($currentgroup) && empty($noattempts)) {
                // we want a particular group and we only want to see students WITH attempts.
                // So join on groups_members and do an inner join on attempts.
                $from  = 'FROM '.$CFG->prefix.'user u JOIN '.$CFG->prefix.'role_assignments ra ON ra.userid = u.id '.
                    'JOIN '.$CFG->prefix.'groups_members gm ON u.id = gm.userid '.
                    'JOIN '.$CFG->prefix.'quiz_attempts qa ON u.id = qa.userid AND qa.quiz = '.$quiz->id;
                $where = ' WHERE ra.contextid ' . $contextlists . ' AND gm.groupid = '. $currentgroup .' AND qa.preview = 0';
            } else if (!empty($currentgroup) && !empty($noattempts)) {
                // We want a particular group and we want to do something funky with attempts
                // So join on groups_members and left join on attempts...
                $from  = 'FROM '.$CFG->prefix.'user u JOIN '.$CFG->prefix.'role_assignments ra ON ra.userid = u.id '.
                    'JOIN '.$CFG->prefix.'groups_members gm ON u.id = gm.userid '.
                    'LEFT JOIN '.$CFG->prefix.'quiz_attempts qa ON u.id = qa.userid AND qa.quiz = '.$quiz->id;
                $where = ' WHERE ra.contextid ' .$contextlists . ' AND gm.groupid = '.$currentgroup;
                if ($noattempts == 1) {
                    // noattempts = 1 means only no attempts, so make the left join ask for only records where the right is null (no attempts)
                    $where .= ' AND qa.userid IS NULL'; // show ONLY no attempts;
                } else {
                    // We are including attempts, so exclude previews.
                    $where .= ' AND qa.preview = 0';
                }
            } else if (empty($currentgroup)) {
                // We don't care about group, and we to do something funky with attempts
                // So do a left join on attempts
                $from  = 'FROM '.$CFG->prefix.'user u JOIN '.$CFG->prefix.'role_assignments ra ON ra.userid = u.id LEFT JOIN '.$CFG->prefix.'quiz_attempts qa ON u.id = qa.userid AND qa.quiz = '.$quiz->id;
                $where = " WHERE ra.contextid $contextlists";
                if (empty($noattempts)) {
                    $where .= ' AND qa.userid IS NOT NULL AND qa.preview = 0'; // show ONLY students with attempts;
                } else if ($noattempts == 1) {
                    // noattempts = 1 means only no attempts, so make the left join ask for only records where the right is null (no attempts)
                    $where .= ' AND qa.userid IS NULL'; // show ONLY students without attempts;
                } else if ($noattempts == 3) {
                    // we want all attempts
                    $from  = 'FROM '.$CFG->prefix.'user u JOIN '.$CFG->prefix.'quiz_attempts qa ON u.id = qa.userid ';
                    $where = ' WHERE qa.quiz = '.$quiz->id.' AND qa.preview = 0';
                } // noattempts = 2 means we want all students, with or without attempts
            }
            $countsql = 'SELECT COUNT(DISTINCT('.sql_concat('u.id', '\'#\'', $db->IfNull('qa.attempt', '0')).')) '.$from.$where;
        } else {
            if (empty($noattempts)) {
                $from   = 'FROM '.$CFG->prefix.'user u JOIN '.$CFG->prefix.'quiz_attempts qa ON u.id = qa.userid ';
                $where = ' WHERE qa.quiz = '.$quiz->id.' AND qa.preview = 0';
                $countsql = 'SELECT COUNT(DISTINCT('.sql_concat('u.id', '\'#\'', $db->IfNull('qa.attempt', '0')).')) '.$from.$where;
            }
        }
        if (!$download) {
            // Add extra limits due to initials bar
            if($table->get_sql_where()) {
                $where .= ' AND '.$table->get_sql_where();
            }

            // Count the records NOW, before funky question grade sorting messes up $from
            if (!empty($countsql)) {
                $totalinitials = count_records_sql($countsql);
                if ($table->get_sql_where()) {
                    $countsql .= ' AND '.$table->get_sql_where();
                }
                $total  = count_records_sql($countsql);

            }

            // Add extra limits due to sorting by question grade
            if($sort = $table->get_sql_sort()) {
                $sortparts    = explode(',', $sort);
                $newsort      = array();
                $questionsort = false;
                foreach($sortparts as $sortpart) {
                    $sortpart = trim($sortpart);
                    if(substr($sortpart, 0, 1) == '$') {
                        if(!$questionsort) {
                            $qid          = intval(substr($sortpart, 1));
                            $select .= ', grade ';
                            $from        .= ' LEFT JOIN '.$CFG->prefix.'question_sessions qns ON qns.attemptid = qa.id '.
                                            'LEFT JOIN '.$CFG->prefix.'question_states qs ON qs.id = qns.newgraded ';
                            $where       .= ' AND (qns.questionid IS NULL OR qns.questionid = '.$qid.')';
                            $newsort[]    = 'answer '.(strpos($sortpart, 'ASC')? 'ASC' : 'DESC');
                            $questionsort = true;
                        }
                    } else {
                        $newsort[] = $sortpart;
                    }
                }

                // Reconstruct the sort string
                $sort = ' ORDER BY '.implode(', ', $newsort);
            }

            // Fix some wired sorting
            if (empty($sort)) {
                $sort = ' ORDER BY attemptindex';
            }
            
            // Now it is time to page the data
            if (!isset($pagesize)  || ((int)$pagesize < 1) ) {
                $pagesize = 10;
            }

            $table->pagesize($pagesize, $total);
        }

        // Fetch the attempts
        if (!empty($from)) { // if we're in the site course and displaying no attempts, it makes no sense to do the query.
            if (!$download) {
                $attempts = get_records_sql($select.$from.$where.$sort,
                                        $table->get_page_start(), $table->get_page_size());
            } else {
                $attempts = get_records_sql($select.$from.$where.$sort);
            }
        } else {
            $attempts = array();
        }

        $format_options = new stdClass;
		$format_options->para = false;
		$format_options->noclean = true;
        $format_options->newlines = false;


        // Build table rows

        if (!$download) {
            $table->initialbars($totalinitials>20);
        }
        if(!empty($attempts) || !empty($noattempts)) {
            if ($attempts) {
            foreach ($attempts as $attempt) {
                if (!$download) {
                    $picture = print_user_picture($attempt->userid, $course->id, $attempt->picture, false, true);
    
                    // uncomment the commented lines below if you are choosing to show unenrolled users and
                    // have uncommented the corresponding lines earlier in this script
                    //if (in_array($attempt->userid, $unenrolledusers)) {
                    //    $userlink = '<a class="dimmed" href="'.$CFG->wwwroot.'/user/view.php?id='.$attempt->userid.'&amp;course='.$course->id.'">'.fullname($attempt).'</a>';
                    //}
                    //else {
                        $userlink = '<a href="'.$CFG->wwwroot.'/user/view.php?id='.$attempt->userid.'&amp;course='.$course->id.'">'.fullname($attempt).'</a>';
                    //}
                    $rowdata = array(
                                  $picture,
                                  $userlink);
    
                    if ($quiz->grade and $quiz->sumgrades) {
                        $rowdata[] = $attempt->sumgrades === NULL ? '<a href="review.php?q='.$quiz->id.'&amp;attempt='.$attempt->id.'">-</a>' : '<a href="review.php?q='.$quiz->id.'&amp;attempt='.$attempt->id.'">'.round($attempt->sumgrades / $quiz->sumgrades * $quiz->grade,$quiz->decimalpoints).'</a>';
                    }
                } else if ($download == 'Excel' || $download == 'ODS' || $download =='CSV') {
                    $rowdata = array($attempt->firstname, $attempt->lastname, $attempt->idnumber,
						        $attempt->institution, $attempt->department, $attempt->email,
								empty($attempt->attempt) ? '-' : userdate($attempt->timestart, $strtimeformat),
                                empty($attempt->timefinish) ? '-' : userdate($attempt->timefinish, $strtimeformat),
                                empty($attempt->attempt) ? '-' : (empty($attempt->timefinish) ? get_string('unfinished', 'quiz') : format_time($attempt->duration))
								);
                    if ($quiz->grade and $quiz->sumgrades) {
                        $rowdata[] = $attempt->sumgrades === NULL ? '-' : round($attempt->sumgrades / $quiz->sumgrades * $quiz->grade,$quiz->decimalpoints); 
                    }
                }
                if(empty($attempt->id)) {
                    foreach($questionids as $questionid) {
                        $rowdata[] = '-';
                    }
                } else {
                    // Restore the question sessions to their most recent states
                    // creating new sessions where required

                    if (!$states = get_question_states($questions, $quiz, $attempt)) {
                        error('Could not restore question sessions');
                    }
                    foreach($questionids as $questionid) {
                        $gradedstateid = get_field('question_sessions', 'newgraded', 'attemptid', $attempt->uniqueid, 'questionid', $questionid);
                        $grade = round(get_field('question_states', 'grade', 'id', $gradedstateid), $quiz->decimalpoints);
                        $responses =  get_question_actual_response($questions[$questionid], $states[$questionid]);
                        $response = implode(', ',$responses);
                        if (!$download) {
                            if ($grade<= 0) {
                                $qclass = 'uncorrect';
                            } elseif ($grade == 1) {
                                $qclass = 'correct';
                            } else {
                                $qclass = 'partialcorrect';
                            }          
                                $responsetext = '<span class="'.$qclass.'">'.format_text("$response", FORMAT_MOODLE, $format_options, $quiz->course).'</span>';
                                $rowdata[]=$responsetext;
                        } else if ($download == 'Excel' || $download == 'ODS' || $download =='CSV') {
                            $rowdata[] = $response;
                        }
                    }
                }
                if (!$download) {    
                    $table->add_data($rowdata);
                } elseif ($download == 'Excel' or $download == 'ODS') {
                    $colnum = 0;
                    foreach($rowdata as $item){
                        $myxls->write($rownum,$colnum,$item);
                        $colnum++;
                    }
                    $rownum++;
                } elseif ($download=='CSV') {
                    $text = implode("\t", $rowdata);
                    echo $text." \n";
                }
            }
		}
    }
    //  Start form
            if (!$download) {
                print_heading_with_help(get_string("responsestitle", "quiz_responses"),"responses", "quiz_responses");
    
                echo '<div id="tablecontainer">';
    
    //  Print table
                $table->print_html();


    //  Print display options
                echo '<div class="controls">';
                echo '<form id="options" action="report.php" method="post">';
                echo '<p class="quiz-report-options">'.get_string('displayoptions', 'quiz').': </p>';
                echo '<input type="hidden" name="id" value="'.$cm->id.'" />';
                echo '<input type="hidden" name="q" value="'.$quiz->id.'" />';
                echo '<input type="hidden" name="mode" value="responses" />';
                echo '<input type="hidden" name="noattempts" value="0" />';
                echo '<table id="responses-options" align="center">';
                echo '<tr align="left">';
                echo '<td><label for="pagesize">'.get_string('pagesize', 'quiz').'</label></td>';
                echo '<td><input type="text" id="pagesize" name="pagesize" size="1" value="'.$pagesize.'" /></td>';
                echo '</tr>';
                echo '<tr align="left">';
                echo '<td colspan="2">';
                $options = array(0 => get_string('attemptsonly','quiz_overview', $course->students));
                if ($course->id != SITEID) {
                    $options[1] = get_string('noattemptsonly', 'quiz_overview', $course->students);
                    $options[2] = get_string('allstudents','quiz_overview', $course->students);
                	$options[3] = get_string('allattempts','quiz_overview');
                }
                choose_from_menu($options,'noattempts',$noattempts,'');
                echo '</td></tr>';
                echo '<tr align="left">';
                echo '<tr><td colspan="2" align="center">';
                echo '<input type="submit" value="'.get_string('go').'" />';
                helpbutton("responsesoptions", get_string("responsesoptions",'quiz_responses'), 'quiz_responses');
                echo '</td></tr></table>';
                echo '</form>';
                echo '</div>';    
                echo "\n";


            if (!empty($attempts)) {
                echo '<table align="center"><tr>';
                unset($options);
                $options["id"] = "$cm->id";
                $options["q"] = "$quiz->id";
                $options["mode"] = "responses";
                $options['sesskey'] = sesskey();
                $options["noheader"] = "yes";
                echo '<td>';
                $options["download"] = "ODS";
                print_single_button("report.php", $options, get_string("downloadods"));
                echo "</td>\n";
                echo '<td>';        
                $options["download"] = "Excel";
                print_single_button("report.php", $options, get_string("downloadexcel"));
                echo "</td>\n";
                echo '<td>';
                $options["download"] = "CSV";
                print_single_button('report.php', $options, get_string("downloadtext"));
                echo "</td>\n";
                echo "<td>";
                helpbutton("responsesdownload", get_string("responsesdownload","quiz"), "quiz");
                echo "</td>\n";
                echo '</tr></table>';
            }
        } 
        elseif ($download == 'Excel' or $download == 'ODS') {
            $workbook->close();
            exit;
        } elseif ($download == 'CSV') {
            exit;
        }

        return true;
    }
}
?>
